//
// Application entrypoint
//
// Author: mla (7-Feb-2020)
// Original: mla (9-Jan-2020)
//

#include "internal.h"

HINSTANCE g_AppInstance;
HWND g_WindowDestroySubscribers[4];

void appNotifyWindowDestroy(HWND hwnd)
{
    int i;

    for(i=0; i<ARRAY_SIZE(g_WindowDestroySubscribers); i++) {
        if(g_WindowDestroySubscribers[i]) {
            SendMessage(g_WindowDestroySubscribers[i],
                        APP_WINDOW_DESTROYED,
                        0,
                        (LPARAM) hwnd);
        }
    }
}

void appSubscribeWindowDestroy(HWND receiver)
{
    int i;

    for(i=0; i<ARRAY_SIZE(g_WindowDestroySubscribers); i++) {
        if(!g_WindowDestroySubscribers[i]) {
            g_WindowDestroySubscribers[i] = receiver;
            break;
        }
    }

    if(i >= ARRAY_SIZE(g_WindowDestroySubscribers)) {
        FatalAppExitW(0, L"Too many window destroy subscribers");
    }
}

void appUnsubscribeWindowDestroy(HWND receiver)
{
    int i;

    for(i=0; i<ARRAY_SIZE(g_WindowDestroySubscribers); i++) {
        if(g_WindowDestroySubscribers[i] == receiver) {
            g_WindowDestroySubscribers[i] = NULL;
            break;
        }
    }

    if(i >= ARRAY_SIZE(g_WindowDestroySubscribers)) {
        FatalAppExitW(0, L"Unsubscribing unknown window");
    }
}

NORETURN void appWin32Error(DWORD code, const WCHAR *func)
{
    WCHAR buffer[256];
    StringCbPrintfW(buffer, sizeof buffer, L"%s: 0x%X", func, code);
    FatalAppExitW(0, buffer);
}

void appWin32Warn(HWND parent, DWORD code, const WCHAR *func)
{
    WCHAR buffer[256];
    StringCbPrintfW(buffer, sizeof buffer, L"%s: 0x%X", func, code);
    MessageBoxW(parent, buffer, NULL, MB_OK | MB_ICONERROR);
}

int WINAPI WinMain(HINSTANCE hinst, HINSTANCE hprev, LPSTR cmdLine, int cmdShow)
{
    MSG msg;
    HWND hwnd;
    INITCOMMONCONTROLSEX icc;
    HACCEL haccel;

    UNREFERENCED_PARAMETER(hprev);
    UNREFERENCED_PARAMETER(cmdLine);
    g_AppInstance = hinst;

    // Initialize Win32 things

    memset(&icc, 0, sizeof icc);
    icc.dwSize = sizeof icc;
    icc.dwICC = ICC_WIN95_CLASSES;
    if(!InitCommonControlsEx(&icc)) {
        appWin32Error(0, L"InitCommonControlsExW");
    }

    haccel = LoadAcceleratorsW(g_AppInstance, L"appAccels");

    // Register classes, etc

    viewInit();
    animInit();
    mainInit();

    // Create main window and enter main loop

    hwnd = mainCreateWindow();
    ShowWindow(hwnd, cmdShow);

    while(GetMessageW(&msg, NULL, 0, 0)) {
        if(haccel == NULL || !TranslateAcceleratorW(hwnd, haccel, &msg)) {
            TranslateMessage(&msg);
            DispatchMessageW(&msg);
        }
    }

    return 0;
}